package com.tanpu.fund.service.impl;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.google.common.collect.Lists;
import com.tanpu.common.enums.BizEnums;
import com.tanpu.common.enums.Constant;
import com.tanpu.common.enums.fund.FundFileTypeEnum;
import com.tanpu.common.model.fund.resq.FundBounsResp;
import com.tanpu.common.model.product.resp.*;
import com.tanpu.common.utils.BigDecimalUtil;
import com.tanpu.fund.entity.generator.*;
import com.tanpu.fund.mapper.generator.*;
import com.tanpu.fund.mapper.generator.custom.FundInfoCustomMapper;
import com.tanpu.fund.service.ProductOrderService;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.jetbrains.annotations.NotNull;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;
import java.util.function.BinaryOperator;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @author: zhoupeng
 * @email: zhoupeng_08@163.com
 */
@Service
public class ProductOrderServiceImpl implements ProductOrderService, Constant {

    @Value("${file.preview.path}")
    private String suffix;

    @Resource
    private FundInfoMapper fundInfoMapper;

    @Resource
    private FundCountMapper fundCountMapper;

    @Resource
    private FundInfoCustomMapper fundInfoCustomMapper;

    @Resource
    private TxFundDistributionMapper txFundDistributionMapper;

    @Resource
    private TxFundNavMapper txFundNavMapper;

    @Override
    public List<FundInfoResp> getFundInfos(List<String> list) {
        FundInfoExample example = new FundInfoExample();
        example.createCriteria().andIdIn(list).andDeleteTagEqualTo(Constant.ZERO_NUM);

        return getFundInfoResps(list, example);
    }

    @NotNull
    private List<FundInfoResp> getFundInfoResps(List<String> list, FundInfoExample example) {
        FundCountExample fundCountExample = new FundCountExample();
        fundCountExample.createCriteria().andFundIdIn(list).andDeleteTagEqualTo(Constant.ZERO_NUM);
        List<FundCount> fundCounts = fundCountMapper.selectByExample(fundCountExample);
        Map<String, FundCount> fundCountMap;
        if (CollectionUtils.isNotEmpty(fundCounts)) {
            fundCountMap = fundCounts.stream().collect(Collectors.toMap(FundCount::getFundId, c -> c));
        } else {
            fundCountMap = null;
        }


        return this.fundInfoMapper.selectByExample(example).stream().map(f -> {
            FundInfoResp resp = new FundInfoResp();
            BeanUtils.copyProperties(f, resp);


            if (fundCountMap != null) {
                FundCount fundCount = fundCountMap.get(f.getId());
                BigDecimal netNav = fundCount.getNetNav();
                resp.setNetNav(BigDecimalUtil.toString(netNav, 4));
                resp.setNet(Net.builder().netDate(fundCount.getEndDate().getTime()).netValue(BigDecimalUtil.toString(netNav, 4)).build());
            }

            resp.setExpenseSubscription(StringUtils.EMPTY);
            resp.setExpenseManage(StringUtils.EMPTY);
            resp.setExpenseAchievement(StringUtils.EMPTY);
            resp.setIfaSubscription(StringUtils.EMPTY);
            resp.setIfaManage(StringUtils.EMPTY);
            resp.setIfaAchievement(StringUtils.EMPTY);

            return resp;
        }).collect(Collectors.toList());
    }

    @Override
    public List<FundInfoResp> getNewFundInfos(List<String> list) {
        FundInfoExample example = new FundInfoExample();
        example.createCriteria().andIdIn(list);

        return getFundInfoResps(list, example);
    }


    @Override
    public List<NetBigDecimal> getFundLastNet(List<String> list) {
        List<FundNav> fundInfoLastNet = fundInfoCustomMapper.getFundInfoLastNet(list);

        List<NetBigDecimal> resList;
        if (CollectionUtils.isNotEmpty(fundInfoLastNet)) {
            resList = fundInfoLastNet.stream().map(item -> NetBigDecimal.builder().netDate(item.getPriceDate().getTime()).netValue(item.getNav()).build()).collect(Collectors.toList());
        } else {
            resList = Lists.newArrayListWithCapacity(0);
        }

        return resList;
    }

    @Override
    public List<FilePreviewResp> getFilePreview(List<String> list) {
        if (CollectionUtils.isEmpty(list)) {
            return new ArrayList<>(0);
        }
        return this.fundInfoCustomMapper.getFilePreviewUrl(list.stream().collect(Collectors.joining("','", "'", "'")))
                .stream().peek(f -> f.setPreviewUrl(suffix + f.getPreviewUrl().substring(f.getPreviewUrl().lastIndexOf("/") + 1, f.getPreviewUrl().lastIndexOf(".")))).collect(Collectors.toList());
    }

    @Override
    public List<FundBounsResp> getFundBounsinfo(List<String> list, String date) {
        LambdaQueryWrapper<TxFundDistribution> queryWrapper = new LambdaQueryWrapper<>();

        queryWrapper.eq(TxFundDistribution::getExDate, date)
                .eq(TxFundDistribution::getDeleteTag, BizEnums.DeleteTag.tag_init);

        if (CollectionUtils.isNotEmpty(list)) {
            queryWrapper.in(TxFundDistribution::getFundCode, list);
        }

        final List<TxFundDistribution> distributionList = this.txFundDistributionMapper
                .selectList(queryWrapper);

        if (CollectionUtils.isNotEmpty(distributionList)) {
            // 查询分红除息日内基金净值
            TxFundNavExample fundNavExample = new TxFundNavExample();
            fundNavExample.createCriteria()
                    .andFundIdIn(distributionList.stream().map(c -> String.valueOf(c.getFundId())).collect(Collectors.toList()))
                    .andPriceDateIn(distributionList.stream().map(TxFundDistribution::getExDate).collect(Collectors.toList()))
                    .andDeleteTagEqualTo(BizEnums.DeleteTag.tag_init);
            final Map<String, List<TxFundNav>> txFundNavMap = this.txFundNavMapper.selectByExample(fundNavExample)
                    .stream().collect(Collectors.groupingBy(TxFundNav::getFundId));

            return distributionList.stream().map(c -> {
                if (!txFundNavMap.containsKey(String.valueOf(c.getFundId()))) {
                    return null;
                }
                final FundBounsResp build = FundBounsResp.builder()
                        .fundId(c.getFundId())
                        .fundCode(c.getFundCode())
                        .exDate(c.getExDate())
                        .distributeDate(c.getDistributeDate())
                        .distribution(c.getDistribution())
                        .build();

                final TxFundNav txFundNav = txFundNavMap.get(String.valueOf(c.getFundId())).stream()
                        .filter(nav -> StringUtils.equals(String.valueOf(c.getFundId()), nav.getFundId())
                        && c.getExDate().getTime() == nav.getPriceDate().getTime()).findFirst().orElse(null);
                if (txFundNav == null) {
                    return null;
                }
                build.setFundNet(txFundNav.getNav());

                return build;
            }).collect(Collectors.toList()).stream().filter(Objects::nonNull).collect(Collectors.toList());
        }

        return new ArrayList<>(0);
    }
}
